#include <windows.h>
#include <string>

#pragma warning ( disable : 4996 ) // sprintf() is deprecated, but we'll ignore the warnings ...

const int ID_TIMER = 1; // For the timer

#define SQUARESIZE 10
#define GRIDX(x) (x)*SQUARESIZE
#define GRIDY(x) (x)*SQUARESIZE

#define BGCOLOR RGB( 0, 100, 200 ) // Background colour

#define GAP 1 // The gap between each 'square' of the snake

#define SPEED(x) 100-5*(x)
#define SNAKELENGTH(x) 7+(x) 

#define SNAKECOLOR RGB( 50, 40, 20 )
#define FOODCOLOR RGB( 80, 180, 80 )

HFONT font;				// A font for writing!
HBRUSH fillBrushSnake;  // Snake colour
HBRUSH fillBrushFood;   // Food colour

RECT posArea = {10, 5, 200, 50}; // RECT for the score position
bool gameOver = false;			 // Not a clue! =D
int score = 0;
int stage = 1;
bool dirChanged = false;		 // Tells us if the direction has changed, to commit the player to their move!
int snakeLength = 0;
bool survivalMode = true;

struct Positions
{
	int xGrid;
	int yGrid;
} Position[40], foodPosition; // Array of positions to make up the snake, position of the food


enum DirectionEnum { up, down, right, left };

DirectionEnum Direction = right; // Start going right


//-------------------------------------------------------------
// Name: DrawSquare()
// Desc: Draws a square of the specified colour
//-------------------------------------------------------------
void DrawSquare(int xGrid, int yGrid, HDC dc, int choice )
{

	static RECT drawPos;
	// Calculate positions for the square, add/minus GAP to leave a small gap between squares (though not necessary)
	drawPos.top = GRIDY(yGrid)+GAP; 
	drawPos.bottom = drawPos.top+SQUARESIZE-GAP;
	drawPos.left = GRIDX(xGrid)+GAP;
	drawPos.right = drawPos.left+SQUARESIZE-GAP; 

	// What are we drawing? Snake or food?
	if ( choice == 1 )
		FillRect( dc, &drawPos, fillBrushSnake );
	else
		FillRect( dc, &drawPos, fillBrushFood );
}

//-------------------------------------------------------------
// Name: MovePosition()
// Desc: Move the snake along whichever direction is needed
//-------------------------------------------------------------
void GenerateFoodPosition()
{
	foodPosition.xGrid = rand()%37+1; // Random number between 1 and 37
	foodPosition.yGrid = rand()%22+3; // random number between 3 and 24
	for ( int i = 0 ; i <= SNAKELENGTH(snakeLength) ; i++ )
	{
		if ( foodPosition.xGrid == Position[i].xGrid && foodPosition.yGrid == Position[i].yGrid )
			GenerateFoodPosition();
	}
}

//-------------------------------------------------------------
// Name: MovePosition()
// Desc: Move the snake along whichever direction is needed
//-------------------------------------------------------------
void MovePosition( int position )
{
	for ( int i = SNAKELENGTH(snakeLength) ; i > 0 ; i-- )
		Position[i] = Position[i-1];

	switch (position)
	{
	case 0: // up
		Position[0].yGrid--;
		break;
	case 1: // down
		Position[0].yGrid++;
		break;
	case 2: // right
		Position[0].xGrid++;
		break;
	case 3: // left
		Position[0].xGrid--;
		break;
	}
}

//-------------------------------------------------------------
// Name: ClearScreen()
// Desc: Clears the game screen
//-------------------------------------------------------------
void ClearScreen( HWND hWnd, HDC hdc )
{
	static HBRUSH clearBrush = CreateSolidBrush( RGB(200,230,200) );
	static RECT clientArea;
	clientArea.top = 30;
	clientArea.bottom = 250;
	clientArea.left = 10;
	clientArea.right = 380;
	FillRect( hdc, &clientArea, clearBrush );
}

//-------------------------------------------------------------
// Name: DrawScreen()
// Desc: Draws the game screen
//-------------------------------------------------------------
void DrawScreen( HWND hWnd )
{
	PAINTSTRUCT PaintStruct;
	HDC hdc = GetDC(hWnd);

	ClearScreen(hWnd, hdc); // Clear the game area
	for ( int i = 0 ; i < SNAKELENGTH(snakeLength) ; i++ ) // Draw the snake
		DrawSquare(Position[i].xGrid, Position[i].yGrid, hdc, 1);

	DrawSquare( foodPosition.xGrid, foodPosition.yGrid, hdc, 2 ); // Draw the food

	
	// Draw the score and the stage--------------------------------------------------
	SelectObject(hdc, font);   // Select our font
	SetBkColor(hdc, BGCOLOR ); // Make the background colour the same as the window's
							   // - Could use SetBkMode( hdc, TRANSPARENT ) to have 
							   // no highlighting on the text, but this way looks better
	SetTextColor( hdc, RGB(255,255,255) ); // Text colour

	static char textString[20];
	sprintf(textString,"Score: %d",score); // Add the score into the string
	
	if ( survivalMode == false )
		DrawText(hdc, textString, -1, &posArea, DT_LEFT); // Draw the score

	if ( survivalMode == true )
	{
		sprintf(textString,"Stage: %d",stage); // Add the stage into the string

		DrawText(hdc, textString, -1, &posArea, DT_RIGHT); // Draw the stage
	}
	//-------------------------------------------------------------------------------
	
	ReleaseDC(hWnd, hdc);
	EndPaint(hWnd, &PaintStruct);
}

//-------------------------------------------------------------
// Name: SetBG()
// Desc: Draws the background (the blue border)
//-------------------------------------------------------------
void SetBG( HWND hWnd )
{
//	PAINTSTRUCT PaintStruct;
	HDC hdc = GetDC(hWnd);

	static HBRUSH clearBrush = CreateSolidBrush( BGCOLOR );
	static RECT clientArea;
	GetClientRect( hWnd, &clientArea );
	FillRect( hdc, &clientArea, clearBrush );		

	ReleaseDC(hWnd, hdc);
//	EndPaint(hWnd, &PaintStruct);
}

//-------------------------------------------------------------
// Name: CheckForCollision()
// Desc: Checks to see if the snake had hit the side or itself
//-------------------------------------------------------------
bool CheckForCollision()
{
	if ( (Position[0].xGrid > 37 || Position[0].xGrid < 1) || (Position[0].yGrid > 24 || Position[0].yGrid < 3) )
		return true;
	for ( int i = 1 ; i < SNAKELENGTH(snakeLength) ; i++ )
		if ( Position[0].xGrid == Position[i].xGrid && Position[0].yGrid == Position[i].yGrid )
			return true;
	return false;

}

//-------------------------------------------------------------
// Name: Restart()
// Desc: Restart the game
//-------------------------------------------------------------
void Restart(HWND hWnd)
{
	SetBG(hWnd);

	// Set the snake in the starting position
	for ( int i = 0 ; i < 16 ; i++ )
	{
		Position[i].xGrid = 16-i;
		Position[i].yGrid = 5;
	}

	score = 0;
	gameOver = false;
	snakeLength = 0;

	Direction = right;

	GenerateFoodPosition();

	int speed = 100-(2*stage-9)*5; // Yes, the speed control is a bit bizzare!
	if ( FAILED( SetTimer(hWnd, ID_TIMER, speed, NULL ) ) )
		MessageBox(hWnd, "Could not SetTimer()!", "Error", MB_OK | MB_ICONEXCLAMATION);

}