#include "drawFunctions.h"

#define WINDOWSIZE_X 410
#define WINDOWSIZE_Y 320

//-- Menu Event IDs --------------
#define ID_FILE_NEWGAME 9001
#define ID_FILE_EXIT 9002
#define ID_LEVEL_1 9003
#define ID_LEVEL_2 9004
#define ID_LEVEL_3 9005
#define ID_LEVEL_4 9006
#define ID_LEVEL_5 9007
#define ID_LEVEL_6 9008
#define ID_LEVEL_7 9009
#define ID_LEVEL_8 9010
#define ID_LEVEL_9 9011
#define ID_LEVEL_SURVIVAL 9012
//--------------------------------


//------------------------------------------------------------------------------------------------
// Name: WndProc()
// Desc: Required function for a Win32 program; basically the event handler, telling the program
//		 what to do on key/button pressed
//------------------------------------------------------------------------------------------------
LRESULT CALLBACK WndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	switch( msg )
	{
	case WM_CREATE:		

		// Create a font
		font = CreateFont(20, NULL, NULL, NULL, NULL, FALSE, NULL, NULL, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, NULL, DEFAULT_QUALITY, DEFAULT_PITCH, "Times New Roman");

		//-- Create a menu -----------------------------------------------------------------
		HMENU hMenu, hSubMenu;

		hMenu = CreateMenu();

        hSubMenu = CreatePopupMenu();
		AppendMenu(hSubMenu, MF_STRING, ID_FILE_NEWGAME, "&New Game");
        AppendMenu(hSubMenu, MF_STRING, ID_FILE_EXIT, "E&xit");
        AppendMenu(hMenu, MF_STRING | MF_POPUP, (UINT)hSubMenu, "&File");

        hSubMenu = CreatePopupMenu();
        
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_1, "&1");
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_2, "&2");
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_3, "&3");
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_4, "&4");
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_5, "&5");
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_6, "&6");
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_7, "&7");
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_8, "&8");
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_9, "&9");
		AppendMenu(hSubMenu, MF_STRING, ID_LEVEL_SURVIVAL, "&Survival");
		AppendMenu(hMenu, MF_STRING | MF_POPUP, (UINT)hSubMenu, "&Level");

        SetMenu(hWnd, hMenu);
		//-----------------------------------------------------------------------------------

		fillBrushSnake = CreateSolidBrush(SNAKECOLOR);
		fillBrushFood = CreateSolidBrush(FOODCOLOR);
		
		Restart(hWnd); // Set all the variables for a new game
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
        {
			case ID_FILE_NEWGAME:
				if ( survivalMode == true )
					stage = 1;
				Restart(hWnd);
		        break;
			case ID_LEVEL_1:
				stage = 1;
				Restart(hWnd);
				survivalMode = false;
				break;
			case ID_LEVEL_2:
				stage = 2;
				Restart(hWnd);
				survivalMode = false;
			break;
			case ID_LEVEL_3:
				stage = 3;
				Restart(hWnd);
				survivalMode = false;
				break;
			case ID_LEVEL_4:
				stage = 4;
				Restart(hWnd);
				survivalMode = false;
				break;
			case ID_LEVEL_5:
				stage = 5;
				Restart(hWnd);
				survivalMode = false;
				break;
			case ID_LEVEL_6:
				stage = 6;
				Restart(hWnd);
				survivalMode = false;
				break;
			case ID_LEVEL_7:
				stage = 7;
				Restart(hWnd);
				survivalMode = false;
				break;
			case ID_LEVEL_8:
				stage = 8;
				Restart(hWnd);
				survivalMode = false;
				break;
			case ID_LEVEL_9:
				stage = 9;
				Restart(hWnd);
				survivalMode = false;
				break;
			case ID_LEVEL_SURVIVAL:
				stage = 1;
				survivalMode = true;
				Restart(hWnd);
				break;
			case ID_FILE_EXIT:            
				KillTimer(hWnd, ID_TIMER);
				PostQuitMessage(0);
				break;
		}
		break;
	case WM_TIMER:
		if ( ! gameOver ) // If it is not game over, move the snake
			MovePosition(Direction);

		dirChanged = false; // Reset this so the player can move turn again

		if ( Position[0].xGrid == foodPosition.xGrid && Position[0].yGrid == foodPosition.yGrid )  // Check if the food has been hit
		{			
			score += stage;
			
			if( survivalMode == false )
				snakeLength++;
			else 
			{
				stage++;

				int newSpeed = 130-2*stage;
				if( newSpeed <= 0 )	newSpeed = 1; // Just incase some freak of nature gets to stage 60! =D

				// Reset the timer to go faster!
				if ( FAILED( SetTimer(hWnd, ID_TIMER, newSpeed, NULL ) ) )
					MessageBox(hWnd, "Could not SetTimer()!", "Error", MB_OK | MB_ICONEXCLAMATION);
				
			}
			GenerateFoodPosition();
		}	

		if ( gameOver == false )
		{
			if ( CheckForCollision() == true )
			{
				gameOver = true;
				
				char textString[40];
				if ( survivalMode == true )
					sprintf(textString,"Game over!\n\nYou got to stage %d",stage); // Add the stage into the string
				else
					sprintf(textString,"Game over!\n\nYou scored %d",score); // Add the stage into the string
				MessageBox( hWnd, textString, "GAME OVER", MB_OK );
			}
			else
				DrawScreen(hWnd);
		}

		break;
	case WM_PAINT:
		SetBG( hWnd );
		DrawScreen( hWnd );
		break;
	case WM_KEYDOWN:
		// dirChanged --> only allow 1 move until the snake is move a square
		switch (wParam)
		{
		case VK_RIGHT:
			if ( Direction != left  && dirChanged == false )
				Direction = right; dirChanged = true;
			break;
		case VK_LEFT:
			if ( Direction != right && dirChanged == false )
				Direction = left; dirChanged = true;
			break;
		case VK_UP:
			if ( Direction != down && dirChanged == false )
				Direction = up; dirChanged = true;
			break;
		case VK_DOWN:
			if ( Direction != up && dirChanged == false )
				Direction = down; dirChanged = true;
			break;
		case VK_ESCAPE:
			KillTimer(hWnd, ID_TIMER);
			PostQuitMessage(0);
			break;
		}
		break;
	case WM_DESTROY:
		KillTimer(hWnd, ID_TIMER);
		PostQuitMessage(0);
		break;
	}
	
	return DefWindowProc( hWnd, msg, wParam, lParam );
}


//------------------------------------------------------------------------------------------------
// Name: WinMain()
// Desc: The most important function in a Win32 program! The analogue to main() in a console program
//------------------------------------------------------------------------------------------------
int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, INT nShowCmd )
{
	MSG msg;

	WNDCLASSEX wc = { sizeof(WNDCLASSEX), NULL, WndProc, NULL, NULL, hInstance, NULL, 
		NULL, (HBRUSH)(COLOR_WINDOW+1), NULL, "wndClass", NULL };

	if( ! RegisterClassEx( &wc ) )
	{
		MessageBox( NULL, "Window Creation Failed!", "Error!", MB_ICONEXCLAMATION | MB_OK );
		return 0;
	}

	// Create the window
	HWND hWnd = CreateWindowEx( WS_EX_CLIENTEDGE, "wndClass", "Snake!", WS_OVERLAPPEDWINDOW, 400, 300, WINDOWSIZE_X, WINDOWSIZE_Y, NULL, NULL, hInstance, NULL );

	if( hWnd == NULL )
	{
		MessageBox(NULL, "Window Creation Failed!", "Error!", MB_ICONEXCLAMATION | MB_OK);
		return 0;
	}

	ShowWindow( hWnd, nShowCmd );
	UpdateWindow( hWnd );

	// Enter the message loop - listening for events and then sending them to WndProc()
	while ( GetMessage( &msg, NULL, NULL, NULL) > 0 )
	{
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}
	
	UnregisterClass( "wndClass", wc.hInstance );
	return 0;
}



